<?php

namespace Service;


use Util\InstanceProvider;

class Organization {

    protected $entityManager;
    protected $organizationFactory;
    protected $environment;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Organization */
    protected $organizationRepository ;

    const ACTIVE = 'A';
    /**
     * @var Structure
     */
    private $structureService;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, \Util\Utilities $environment,
                                \Factory\Organization $organizationFactory,Structure $structureService) {
        $this->entityManager = $entityManager;
        $this->organizationFactory = $organizationFactory;
        $this->environment = $environment;
        $this->structureService = $structureService;
        $this->organizationRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Organization');
    }

    public function getCurrentOrganizationData(){
        return $this->organizationRepository->getOrganization($this->environment->getCurrentOrganizationId());
    }

    public function createOrganization($name, $foundation, $document, $description) {
        $entity = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Organization')
            ->findOneBy(array("name" => $name));

        if(empty($entity)){
            $entity = $this->organizationFactory->createOrganization($name, $foundation, self::ACTIVE, $document, $description);
            $this->entityManager->persist($entity);
            $this->entityManager->flush();
        } else {
            throw new \Exception('já existe uma organização com o nome ' . $entity->getName() . '.', 1);
        }

        return $entity;
    }

    public function updateOrganization($organization, $name, $foundation, $status, $document, $description, $creator = null) {
        $organizationModel = '\Zeedhi\DevOps\Model\Projectmanagement\Entities\Organization';
        if(!($organization instanceOf $organizationModel)){
            $organization = $this->entityManager->find($organizationModel, $organization);
        }
        $entity = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Organization')
            ->findOneBy(array("name" => $name));

        if(empty($entity) || $entity->getId() === $organization->getId()){
            $entity = $this->organizationFactory->updateOrganization($organization, $name, $foundation, $status, $document, $description, $creator);
            $this->entityManager->persist($entity);
            $this->entityManager->flush();
        } else{
            throw new \Exception('já existe uma organização com o nome ' . $entity->getName() . '.', 1);
        }

        return $entity;
    }

    public function updateCurrentOrganization($name, $foundation, $document, $description, $managers, $calendarId,
                                              $calendarDefaultId, $structureId, $timeSheetRequired = 'N',
                                              $replicateTimeSheetRequired = false){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Organization $organization */
            $organization = $this->environment->getCurrentOrganization();
            /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Structure $structure */
            $structure = InstanceProvider::getRepository()->getStructure()->find($structureId);
            $structure->setName($name);
            $structure->setTimeSheetRequired($timeSheetRequired);
            $this->entityManager->persist($structure);
            $this->structureService->saveEditCalendar($calendarId, $calendarDefaultId, $structureId);
            $this->structureService->setManager($structureId, $managers);
            $organization = $this->updateOrganization($organization, $name, $foundation, $organization->getStatus(),
                $document, $description);
            $this->structureService->setTimeSheetRequired($replicateTimeSheetRequired, $structure,$timeSheetRequired);
            $this->entityManager->flush();
            $connection->commit();
        }catch (\Exception $e){
            $connection->rollBack();
            throw $e;
        }
        return array(
            'userName' => $this->environment->getCurrentUser()->getName(),
            'organizationName' => $organization->getName()
        );
    }

    public function deleteOrganization(\Zeedhi\DevOps\Model\Projectmanagement\Entities\Organization $organization) {
        $this->entityManager->remove($organization);
        $this->entityManager->flush();
        return $organization;
    }
}
