<?php
namespace Service;


use Doctrine\ORM\EntityManager;
use Util\Utilities;
use \Zeedhi\DevOps\Util\Utilities AS DevOpsUtilities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Util\Messages;
use Zeedhi\Framework\Controller\Exception;
use Util\InstanceProvider;

class ResourceProject {

    protected  $entityManager;
    protected  $environment;
    protected  $repositoryService;
    protected  $directoryFactory;
    /** @var Repositories\ResourceProject $resourceProject*/
    protected $resourceProjectRepository;

    /** @var Repositories\Project */
    protected $projectRepository;

    /** @var Repositories\Resource */
    protected $resourceRepository;

    /** @var Repositories\Repository*/
    protected $repositoryRepository;

    /** @var Repositories\Directory*/
    protected $directoryRepository;
    
    const MANAGER = Entities\ResourceProject::MANAGER;
    const SPONSOR = Entities\ResourceProject::SPONSOR;
    const WORK = Entities\ResourceProject::WORK;

    public function __construct(EntityManager $entityManager, Utilities $environment, \Service\Repository $repositoryService,
                                \Factory\Directory $directoryFactory, \Factory\ResourceProject $resourceProjectFactory)
    {

        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->repositoryService = $repositoryService;
        $this->directoryFactory = $directoryFactory;
        $this->resourceProjectFactory = $resourceProjectFactory;
        $this->resourceProjectRepository = $entityManager->getRepository(Utilities::ENTITY_DIR . 'ResourceProject');
        $this->projectRepository = $entityManager->getRepository(Utilities::ENTITY_DIR . 'Project');
        $this->resourceRepository = $entityManager->getRepository(Utilities::ENTITY_DIR . 'Resource');
        $this->repositoryRepository = $entityManager->getRepository(Utilities::ENTITY_DIR . 'Repository');
        $this->directoryRepository = $entityManager->getRepository(Utilities::ENTITY_DIR . 'Directory');
        \Util\InstanceProvider::getLog()->setLog('instanciando ResourceProject service',false,2);
    }

    public function updateProjectPartner(   Entities\ResourceProject $resourceProject, Entities\Resource $resource,
                                            Entities\Project $project,
                                            $type){
        $resourceProject = $this->resourceProjectFactory->updateResourceProject($resourceProject, $resource, $project, $type);
        $this->entityManager->persist($resourceProject);
        $this->entityManager->flush();
    }

    /**
     * @param Entities\Resource[] $resources
     * @param Entities\Project $project
     * @param Entities\Resource[] $oldManagerIds
     * @param Entities\Resource[] $newManagerIds
     * @throws \Exception
     */
    public function setManagersProject(array $resources,Entities\Project $project, array $oldManagerIds,array $newManagerIds){
        $type = ResourceProject::MANAGER;
        $isRemoveOldManagers = !empty($oldManagerIds);
        $isAddNewManagers = !empty($newManagerIds);
        $this->updateProjectPartnerSponsorOrManager($resources, $project, $type );
        $projectChildrenIds = DevOpsUtilities::getAllProjectLessIdInArray($project->getId());
        foreach ($projectChildrenIds as $projectChildrenId) {
            if($isRemoveOldManagers){
                foreach ($oldManagerIds as $oldManagerId) {
                    $this->removeManager($projectChildrenId, $oldManagerId);
                }
            }

            if($isAddNewManagers){
                foreach ($resources as $resource) {
                    $this->createResourceProject($resource, $projectChildrenId, $type);
                }
            }
        }
    }

    public function updateProjectPartnerSponsorOrManager( $resources,Entities\Project $project,
                                                         $type, $merge = true,$recursive = false)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            if (!is_array($resources)) {
                $resources = array($resources);
            }
            $resourceIds = array();
            /** @var Entities\Resource[] $resources */
            foreach ($resources as $resource) {
                $resourceIds[] = $resource->getId();
                $this->createResourceProject($resource->getId(), $project->getId(), $type);
            }

            /** @var Entities\ResourceProject[] $resourceProjects */
            $resourceProjects = $this->resourceProjectRepository->findBy(array(
                "project" => $project, "type" => $type));

            // If there is already an old manager/sponsor, let's transform it to a resource of Worker type
            foreach ($resourceProjects as $resourceProject) {
                if (!empty($resourceProject) && (!in_array($resourceProject->getResourceC()->getId(),$resourceIds))) {
                    if ($merge) {
                        $resourceProject->setType(self::WORK);
                        $this->entityManager->persist($resourceProject);
                    } else {
                        $this->entityManager->remove($resourceProject);
                    }
                    $this->entityManager->flush();
                }
            }


            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param Entities\Project $project
     * @return Entities\Resource[]
     */
    public function getManagersByProject(Entities\Project $project){
        $resourceProjects = $this->resourceProjectRepository->findBy(array(
            "project" => $project, "type" => self::MANAGER));
        return Utilities::getArrayByArrayObjectWithGet($resourceProjects,array('getResourceC'));

    }

    public function createResourceProjectFast($resource, $project, $type = self::WORK)
    {
        $entity = $this->resourceProjectFactory->createResourceProject($resource, $project, $type);
        $this->entityManager->persist($entity);
    }

    public function createResourceProject($resource, $project, $type = self::WORK, $recursive = false,$onRecursive = false)
    {

        if (empty($type)) {
            $type = self::WORK;
        }

        $project = $this->projectRepository->find($project);
        $resource = $this->resourceRepository->find($resource);
        if ($recursive) {

            /** @var Entities\Project[] $children */
            $children = $this->projectRepository->getProjectChildrenNotConcludedOrCanceled($project->getId());

            foreach ($children as $child) {
                $this->createResourceProject($resource->getId(), $child, self::WORK, false, true);
            }
        }
        $entity = $this->resourceProjectRepository->findOneBy(array('resourceC' => $resource, 'project' => $project));
        if (empty($entity)) {
            $entity = $this->resourceProjectFactory->createResourceProject($resource, $project, $type);
        } else {
            if (!$onRecursive) {
                $entity->setType($type);
            }
        }
        $this->entityManager->persist($entity);
        $this->entityManager->flush();
        return $entity;

    }
    public function deleteResourceProjectById($idResourceProject, $recursive = false) {
        $connection = $this->entityManager->getConnection();
        $errorMessages = array();
        try{
            $connection->beginTransaction();
            $resourceProjectRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\ResourceProject');
            if(!empty($idResourceProject)){
                /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\ResourceProject $entity */
                $entity = $resourceProjectRepository->find($idResourceProject);
                $project = $entity->getProject();
                $resourceId = $entity->getResourceC()->getId();
                if($recursive){
                    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Project $projectRepository */
                    $projectRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Project');
                    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Project[] $children */
                    $children = $projectRepository->getProjectChildrenNotConcludedOrCanceled($project->getId());
                        foreach ($children as $child) {
                            $resourceProjectChild = $resourceProjectRepository
                                ->findOneBy(array('project' => $child, 'resourceC' => $resourceId));
                            if(!empty($resourceProjectChild)){
                                $errorMessages = array_merge(   $errorMessages,
                                                                $this->deleteResourceProjectById($resourceProjectChild->getId(), false));
                            }
                        }

                }
                if($entity->getType() === 'M' || $entity->getType() === 'S' ){
                    array_push( $errorMessages, ' ' . $entity->getResourceC()->getName() . ' do projeto ' .
                                                $project->getName());
                }
                else{
                    $this->entityManager->remove($entity);
                    $this->entityManager->flush();
                }
            }
            $connection->commit();
        }
        catch(\Exception $e){
            $connection->rollBack();
            throw $e;
        }

        return $errorMessages;
    }

    public function setResourceProjectPaths($repositoryId, $directory, $resourceProjectId, $projectId, $repositoryStatus, $taskId = null){
        $path = array();
        $organization = $this->environment->getCurrentOrganization();
        $resourceProjectEntity = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\ResourceProject', $resourceProjectId);
        $resourceProjectFactory = new \Factory\ResourceProject($this->entityManager);

        if($directory !== null){
            $resourceProject = $resourceProjectFactory->setLocalDirectory($resourceProjectEntity, $directory);
            $this->entityManager->persist($resourceProject);
            $this->entityManager->flush();

            $path['DIRECTORY'] = $directory;
        }
        if($repositoryId !== null){
            $repository = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Repository', $repositoryId);
            $task = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Task', $taskId);
            $taskBranchesFactory = new \Factory\TaskBranches($this->entityManager);
            $taskBranches = $taskBranchesFactory->createTaskBranch($organization, $task, $repositoryStatus);
            $this->entityManager->persist($taskBranches);
            $this->entityManager->flush();

            $path['REPOSITORY'] = $repository->getUrl();
        }
        return $path;
    }

    public function getAllResourceProjectsPaths(){
        $user = $this->environment->getCurrentUserId();
        $dataSet =   $this->resourceProjectRepository->getProjectPaths($user);
        return Utilities::fixParentsForTreeGrid($dataSet);
    }

    public function getAllResourcePaths($projectId){
        $user = $this->environment->getCurrentUserId();
        $projectId = preg_replace("/[^0-9]/", "",$projectId);
        $dataSet =    $resourceProjectRepository = $this->resourceProjectRepository
            ->getResourceProjectPaths($user,$projectId);

        return $dataSet;
    }

    public function saveDirectory($repository, $directory, $user = null, $directoryId = null, $linuxDirectory = null){
        if($user){
            $user = $this->resourceRepository->find($user);
        }else{
            $user = $this->environment->getCurrentUser();
        }
        /** @var Entities\Repository $repository */
        $repository = $this->repositoryRepository->find($repository);

        if($directoryId == null){
            $directory = $this->directoryFactory->createDirectory($directory, $user, $repository, $linuxDirectory);
        } else{
            /** @var Entities\Directory $entity */
            $entity = $this->directoryRepository->find($directoryId);
            $directory = $this->directoryFactory->updateDirectory($entity, $directory, $user, $repository, $linuxDirectory);
        }
        $this->entityManager->persist($directory);
        $this->entityManager->flush();
        return $directory;
    }

    public function copyDirectories($selectedPaths, $project){
        $connection = $this->entityManager->getConnection();
        try{
            $connection->beginTransaction();
            foreach($selectedPaths as $paths) {
                $directory = $paths['LOCAL_DIRECTORY'];
                $linuxDirectory = $paths['LINUX_DIRECTORY'];
                $repositoryUrl = $paths['REPOSITORY_URL'];
                $this->copyDirectory($repositoryUrl, $directory, $project, $linuxDirectory);
            }
            $connection->commit();
        }catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function copyDirectory($repositoryUrl, $directory, $projectId, $linuxDirectory){
            $user = $this->environment->getCurrentUser();
            /** @var Entities\Repository $repository */
            $project = $this->projectRepository->find($projectId);
            $repository = $this->repositoryRepository->findOneBy(array("url" => $repositoryUrl, "project" => $project));
            if($repository) {
                $oldDirectory = $this->directoryRepository->findOneBy(array("repository" => $repository, "resource" => $user));
                if ($oldDirectory == null) {
                    $directory = $this->directoryFactory->createDirectory($directory, $user, $repository, $linuxDirectory);
                } else {
                    $directoryId = $oldDirectory->getId();
                    /** @var Entities\Directory $entity */
                    $entity = $this->directoryRepository->find($directoryId);
                    $directory = $this->directoryFactory->updateDirectory($entity, $directory, $user, $repository, $linuxDirectory);
                }
                $this->entityManager->persist($directory);
                $this->entityManager->flush();
            }
            return $directory;
    }

    public function verifyDirectory($repositoryUrl, $projectId){
            $success = true;
            /** @var Entities\Repository $repository */
            $project = $this->projectRepository->find($projectId);
            $repository = $this->repositoryRepository->findOneBy(array("url" => $repositoryUrl, "project" => $project));
            if($repository == null){
                $success = false;
            }
            return $success;
    }


    /**
     * @param $selectedPaths
     * @param $project
     * @return array
     */
    public function verifyProjectPaths($selectedPaths, $project)
    {
        $failedProjectsPaths = array();
        $successfulProjectPaths = array();
        foreach ($selectedPaths as $paths) {
            $repositoryUrl = $paths['REPOSITORY_URL'];
            $success = $this->verifyDirectory($repositoryUrl, $project);
            if ($success) {
                array_push($successfulProjectPaths, $repositoryUrl);
            } else {
                array_push($failedProjectsPaths, $repositoryUrl);
            }
        }
        return array($successfulProjectPaths, $failedProjectsPaths);
    }

    public function deleteDirectory($id) {
        $directoryRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Directory');
        /**@var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource*/
        $directory = $directoryRepository->findOneBy(array('id'=>$id));
        $this->entityManager->remove($directory);
        $this->entityManager->flush();
    }

    /**
     * @param $projectChildrenId
     * @param $oldManager
     */
    protected function removeManager($projectChildrenId, $oldManager)
    {
        $type = ResourceProject::MANAGER;
        $managers = $this->resourceProjectRepository->findBy(array('type' => $type, 'project' => $projectChildrenId));
        /** @var Entities\ResourceProject $resourceProject */
        $resourceProject = $this->resourceProjectRepository->findOneBy(array('resourceC' => $oldManager,
            'project' => $projectChildrenId));

        if ($resourceProject && (
                ($resourceProject->getType() == $type && count($managers) > 1) || ($resourceProject->getType() != $type)
            )
        ) {
            $resourceProject->setType(Entities\ResourceProject::WORK);
            $this->entityManager->persist($resourceProject);
            $this->entityManager->flush();
        }
    }

} 